package api.common.util;

import api.common.pojo.annotation.WChildren;
import api.common.pojo.annotation.WId;
import api.common.pojo.annotation.WParentId;
import api.common.pojo.vo.system.DictVO;
import com.fasterxml.jackson.databind.JsonNode;
import org.springframework.beans.BeanUtils;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.security.PublicKey;
import java.util.*;
import java.util.function.BiConsumer;
import java.util.function.Function;

public class ObjectUtil {

    /**
     * 空值判断
     *
     * @param obj
     * @return
     */
    public static boolean isNull(Object obj) {
        boolean isNullFlag = true;
        if (obj != null) {
            if (obj instanceof List) {
                isNullFlag = isNull((List) obj);
            } else if (obj instanceof Set) {
                isNullFlag = isNull((Set) obj);
            } else if (obj instanceof Object[]) {
                isNullFlag = isNull((Object[]) ((Object[]) obj));
            } else if (obj instanceof Map) {
                isNullFlag = isNull((Map) obj);
            } else if (obj instanceof String) {
                isNullFlag = isNull((String) obj);
            } else if (obj instanceof Integer) {
                isNullFlag = isNull((Integer) obj);
            } else if (obj instanceof Boolean) {
                isNullFlag = isNull((Boolean) obj);
            } else if (obj instanceof StringBuffer) {
                isNullFlag = isNull((StringBuffer) obj);
            } else if (obj instanceof JsonNode) {
                isNullFlag = isNull((JsonNode) obj);
            } else {
                isNullFlag = false;
            }
        }
        return isNullFlag;
    }

    public static boolean isNotNull(Object obj) {
        return !isNull(obj);
    }

    public static boolean isNull(List<?> list) {
        return list == null || list.size() == 0 || list.get(0) == null;
    }

    public static boolean isNull(Set<?> set) {
        return set == null || set.size() == 0;
    }

    public static boolean isNull(Object[] objects) {
        return objects == null || objects.length == 0;
    }

    public static boolean isNull(Map<?, ?> map) {
        return map == null || map.isEmpty();
    }

    public static boolean isNull(String str) {
        return str == null || "".equals(str.trim()) || "null".equalsIgnoreCase(str.trim()) || "undefined".equalsIgnoreCase(str.trim());
    }

    public static boolean isNull(Integer integer) {
        return integer == null;
    }

    public static boolean isNull(Boolean bol) {
        return bol == null;
    }

    public static boolean isNull(StringBuffer buffer) {
        return buffer == null || buffer.length() == 0;
    }

    public static boolean isNull(JsonNode jsonNode) {
        return jsonNode.isNull() || jsonNode.isEmpty();
    }

    public static List<DictVO> DictListToTree(List<DictVO> list) {
        List<DictVO> treeList = new ArrayList<DictVO>();
        for (DictVO dictVO: list) {
            if("0".equals(dictVO.getPid())){
                treeList.add(findChildren(dictVO,list));
            }
        }
        return treeList;
    }

    private static DictVO findChildren(DictVO dictVO, List<DictVO> list) {
        for (DictVO node: list) {
            if(dictVO.getId().equals(node.getPid())){
                if(dictVO.getChildren() == null || dictVO.getChildren().size() == 0){
                    dictVO.setChildren(new ArrayList<DictVO>());
                }
                dictVO.getChildren().add(findChildren(node,list));
            }
        }
        return dictVO;
    }

    public static void voToPo(Object source, Object target){
        BeanUtils.copyProperties(source,target);
    }

    /**
     * list转tree结构，
     * @param list List<T>类型数据源
     * @param rootId tree的根节点id
     * @param getId <T>类型获取id的方法
     * @param getPid <T>类型获取父id的方法
     * @param getChildren <T>类型获取子对象的方法
     * @param setChildren <T>类型设置子对象的方法
     * @param <T> 该对象满足父子关系的相关字段：id 和 父id
     * @param <String> id的数据类型为String
     * @return 树形结构的List<T>，多根型
     */
    public static <T,String> List<T> listToTree(List<T> list, String rootId, Function<T, String> getId , Function<T, String> getPid, Function<T, List<T>> getChildren, BiConsumer<T,List<T>> setChildren) {
        List<T> treeList = new ArrayList<T>();
        for (T node : list) {
            if (rootId.equals(getPid.apply(node))) {
                treeList.add(findChildren(node, list,getId,getPid,getChildren,setChildren));
            }
        }
        return treeList;
    }

    /**
     * list转tree递归调用
     */
    private static <T,String> T findChildren(T parent, List<T> list, Function<T, String> getId, Function<T, String> getPid, Function<T, List<T>> getChildren , BiConsumer<T,List<T>> setChildren) {
        for (T node : list)
            if (getId.apply(parent).equals(getPid.apply(node))) {
                if (getChildren.apply(parent) == null || getChildren.apply(parent).size() == 0) {
                    setChildren.accept(parent, new ArrayList<T>());
                }
                getChildren.apply(parent).add(findChildren(node, list, getId, getPid, getChildren,setChildren));
            }
        return parent;
    }

    /**
     * list转tree结构，<T>类型需要在属性上添加注解，标识树形结构：@WId 、@WParentId、 @WChildren;
     * @param list
     * @param <T>
     * @return
     */
    public static <T> List<T> listToTree(List<T> list){
        if(isNull(list)){
            return new LinkedList<T>();
        }
        return listToTree(list,"0");
    }

    public static <T> List<T> listToTree(List<T> list,String root) {
        try {
            List<T> treeList = new LinkedList<T>();
            Class<?> tClass = null;
            for (T t : list) {
                tClass = t.getClass();
                break;
            }
            //反射解析Getter和Setter
            Field[] fields = tClass.getDeclaredFields();
            Method getId = null;
            Method getParentId = null;
            Method getChildren = null;
            Method setChildren = null;
            for (Field field:fields) {
                String fieldName= field.getName();
                String firstChar = fieldName.substring(0,1);
                if(field.isAnnotationPresent(WId.class)){
                    getId = tClass.getMethod("get" + fieldName.replaceFirst(firstChar,firstChar.toUpperCase()));
                };
                if(field.isAnnotationPresent(WParentId.class)){
                    getParentId = tClass.getMethod("get" + fieldName.replaceFirst(firstChar,firstChar.toUpperCase()));
                };
                if(field.isAnnotationPresent(WChildren.class)){
                    getChildren = tClass.getMethod("get" + fieldName.replaceFirst(firstChar,firstChar.toUpperCase()));
                    setChildren = tClass.getMethod("set" + fieldName.replaceFirst(firstChar,firstChar.toUpperCase()),List.class);
                };
            }
            //解析列表
            for (T node : list) {
                if (root.equals((String)getParentId.invoke(node))) {
                    treeList.add(findChildren(node, list,getId,getParentId,getChildren,setChildren));
                }
            }
            return treeList;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return list;
    }

    /**
     * list转tree递归调用
     */
    private static <T> T findChildren(T parent, List<T> list, Method getId, Method getParentId, Method getChildren , Method setChildren) throws InvocationTargetException, IllegalAccessException {
        for (T node : list){
            String id = (String)getId.invoke(parent);
            String parentId = (String)getParentId.invoke(node);
            if(id.equals(parentId)){
                List<T> children = (List<T>) getChildren.invoke(parent);
                if(ObjectUtil.isNull(children)){
                    children = new ArrayList<T>();
                    setChildren.invoke(parent,children);
                }
                children.add(findChildren(node, list,getId,getParentId,getChildren,setChildren));
            }
        }
        return parent;
    }
}
