package com.css.simulation.resource.scheduler.manager;

import api.common.pojo.constants.DictConstants;
import api.common.pojo.dto.ProjectMessageDTO;
import api.common.util.*;
import com.css.simulation.resource.scheduler.mapper.AutoSubProjectMapper;
import com.css.simulation.resource.scheduler.mapper.IndexMapper;
import com.css.simulation.resource.scheduler.mapper.ManualProjectMapper;
import com.css.simulation.resource.scheduler.mapper.TaskMapper;
import com.css.simulation.resource.scheduler.pojo.po.IndexTemplatePO;
import com.css.simulation.resource.scheduler.pojo.po.LeafIndexPO;
import com.css.simulation.resource.scheduler.pojo.po.ProjectPO;
import com.css.simulation.resource.scheduler.pojo.po.TaskPO;
import com.css.simulation.resource.scheduler.pojo.to.PrefixTO;
import com.css.simulation.resource.scheduler.pojo.to.ScoreTO;
import com.css.simulation.resource.scheduler.util.MinioUtil;
import com.css.simulation.resource.scheduler.util.ProjectUtil;
import com.css.simulation.resource.scheduler.util.RedisUtil;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import io.minio.MinioClient;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.sshd.client.SshClient;
import org.apache.sshd.client.session.ClientSession;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.*;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

@Component
@Slf4j
public class TaskManager {

    @Value("${scheduler.linux-path.pod-yaml-directory}")
    String podYamlDirectory;
    @Value("${minio.bucket-name}")
    String bucketName;
    @Value("${scheduler.linux-path.score-py}")
    String pyPath;
    @Value("${scheduler.linux-path.temp}")
    String linuxTempPath;
    @Value("${simulation-cloud.client-id}")
    String clientId;
    @Value("${simulation-cloud.client-secret}")
    String clientSecret;
    @Value("${simulation-cloud.token-uri}")
    String tokenUri;
    @Value("${simulation-cloud.evaluation-level-uri}")
    String evaluationLevelUri;
    @Value("${scheduler.minio-path.project-result}")
    String resultPathMinio;
    @Value("${spring.kafka.hostname}")
    String hostnameKafka;
    @Value("${spring.kafka.username}")
    String usernameKafka;
    @Value("${spring.kafka.password}")
    String passwordKafka;
    @Value("${spring.kafka.delete-command}")
    String kafkaDeleteCommand;
    @Resource
    StringRedisTemplate stringRedisTemplate;
    @Resource
    TaskMapper taskMapper;
    @Resource
    MinioClient minioClient;
    @Resource
    ManualProjectMapper manualProjectMapper;
    @Resource
    AutoSubProjectMapper autoSubProjectMapper;
    @Resource
    TaskIndexManager taskIndexManager;
    @Resource
    IndexMapper indexMapper;
    @Resource
    CloseableHttpClient closeableHttpClient;
    @Resource
    RequestConfig requestConfig;
    @Resource
    ProjectUtil projectUtil;

    @SneakyThrows
    @Transactional
    public boolean isProjectCompleted(PrefixTO redisPrefix, String projectId, String taskId, String state, String podName) {
        String nodeName = projectUtil.getNodeNameOfPod(podName);
        if ("Running".equals(state)) {  // 运行中的 pod 无需删除
            // 将运行中的任务的 pod 名称放入 redis
            stringRedisTemplate.opsForValue().set(redisPrefix.getTaskPodKey(), podName);
            taskTick(taskId); // 刷新一下心跳
            log.info("TaskManager--state 修改任务 " + taskId + " 的状态为 " + state + "，pod 名称为：" + podName);
            taskMapper.updateStateWithStartTime(taskId, state, TimeUtil.getNowForMysql());
            return false;
        } else { // 结束的 pod 都直接删除，并判断项目是否完成
            // -------------------------------- 处理状态 --------------------------------
            // TODO 暂时不用重试操作
            try {
                log.info("TaskManager--state 修改任务 " + taskId + "的状态为 " + state + "，pod 名称为：" + podName + "，并删除 pod。");
                if ("Aborted".equals(state)) {
                    String minioPathOfErrorLog = resultPathMinio + projectId + "/" + taskId + "error.log";
                    boolean objectExist = MinioUtil.isObjectExist(minioClient, bucketName, minioPathOfErrorLog);
                    String targetEvaluate;
                    if (objectExist) {
                        String errorString = MinioUtil.downloadToString(minioClient, bucketName, minioPathOfErrorLog);
                        String[] lines = errorString.split("\n");
                        StringBuilder errorMessage = new StringBuilder();
                        for (String line : lines) {
                            if (line.startsWith("Original Error")) {
                                errorMessage.append(line).append("\n");
                            }
                            if (line.startsWith("Possible Cause")) {
                                errorMessage.append(line);
                                break;
                            }
                        }
                        targetEvaluate = errorMessage.toString();
                    } else {
                        targetEvaluate = DictConstants.TASK_ERROR_REASON_2;
                    }
                    taskMapper.updateFailStateWithStopTime(taskId, state, TimeUtil.getNowForMysql(), targetEvaluate);
                } else if ("Terminated".equals(state)) {
                    taskMapper.updateFailStateWithStopTime(taskId, state, TimeUtil.getNowForMysql(), DictConstants.TASK_ERROR_REASON_3);
                } else if ("PendingAnalysis".equals(state)) {
                    taskMapper.updateSuccessStateWithStopTime(taskId, state, TimeUtil.getNowForMysql());
                    // 创建视频和生成
                    //1 获取两个 csv 的目录，和 xodr 和 osgb 三个路径
                    String csvDirectory = linuxTempPath + "video/" + projectId + "/" + taskId + "/";
                    String xodrPath = FileUtil.listAbsolutePathByFiletype(csvDirectory, ".xodr").get(0);
                    String osgbPath = FileUtil.listAbsolutePathByFiletype(csvDirectory, ".osgb").get(0);
                    String xoscPath = "";
//                    generateService.generateXosc;
                    //2 根据获得的 xoscPath 调用 linux 命令生成视频
                    String cmd = LinuxUtil.execute("cmd");
                    // 根据 task 对应的场景的 xodr 文件
                    //2 输入到工具类中获取到图片
                    //3 将图片生成视频
                    //3 将视频上传到 minio
                }
                // -------------------------------- 判断项目是否结束 --------------------------------
                ProjectMessageDTO projectMessageDTO = JsonUtil.jsonToBean(stringRedisTemplate.opsForValue().get(redisPrefix.getProjectRunningKey()), ProjectMessageDTO.class);
                int taskTotal = projectMessageDTO.getTaskTotal();
                int taskCompleted = projectMessageDTO.getTaskCompleted();
                Integer currentParallelism = projectMessageDTO.getCurrentParallelism();
                log.info("TaskManager--isProjectCompleted 项目 " + projectId + " 完成进度为：" + (taskCompleted + 1) + "/" + taskTotal);
                if (taskCompleted + 1 == taskTotal) {
                    return true;
                } else {    // 项目没有完成
                    projectMessageDTO.setTaskCompleted(taskCompleted + 1);  // 增加已完成任务数
                    stringRedisTemplate.opsForValue().set(redisPrefix.getProjectRunningKey(), JsonUtil.beanToJson(projectMessageDTO));
                    if (taskTotal - taskCompleted <= currentParallelism) {
                        // 如果 taskTotal - taskCompleted 小于 currentParallelism ，则不启动下一个 pod，删除当前 pod 并归还一个并行度
                        projectUtil.deletePod(podName);
                        projectUtil.addOneParallelismToNode(nodeName);
                    } else {
                        projectUtil.createNextPod(projectId, nodeName, podName);  // 删除上一个 pod，并启动下一个 pod
                    }
                    return false;
                }
            } catch (Exception exception) {
                log.info("TaskManager--isCompleted pod " + podName + " 已经被手动删除，该项目可能已被终止。", exception);
                return false;
            }
        }
    }


    public void prepareScore(String projectRunningKey) {
        stringRedisTemplate.delete(projectRunningKey);
    }

    @SneakyThrows
    @Transactional
    public void score(PrefixTO redisPrefix, String userId, String projectId, String projectType, ClientSession session) {
        // -------------------------------- 打分 --------------------------------
        ProjectPO projectPO = null;
        if (DictConstants.PROJECT_TYPE_MANUAL.equals(projectType)) {
            projectPO = manualProjectMapper.selectById(projectId);
        } else if (DictConstants.PROJECT_TYPE_AUTO_SUB.equals(projectType)) {
            projectPO = autoSubProjectMapper.selectById(projectId);
        }
        String packageId = projectPO.getScenePackageId();  // 场景测试包 id，指标的rootId
        List<TaskPO> taskList = taskMapper.selectTaskListByProjectId(projectId);  // 所有任务信息
        if (CollectionUtil.isEmpty(taskList)) {
            log.error("TaskManager--score 项目 " + projectId + " 下没有查询到任务！");
            return;
        }
        indexMapper.deleteFirstByProjectId(projectId);
        indexMapper.deleteLastByProjectId(projectId);
        //1 查询场景包对应指标
        String allIndexTemplateListJson = stringRedisTemplate.opsForValue().get(redisPrefix.getProjectRunningKey() + ":package:" + packageId + ":all");
        List<IndexTemplatePO> allIndexTemplateList = JsonUtil.jsonToList(allIndexTemplateListJson, IndexTemplatePO.class);
        String leafIndexTemplateListJson = stringRedisTemplate.opsForValue().get(redisPrefix.getProjectRunningKey() + ":package:" + packageId + ":leaf");
        List<IndexTemplatePO> leafIndexTemplateList = JsonUtil.jsonToList(leafIndexTemplateListJson, IndexTemplatePO.class);
        log.info("TaskManager--score 共有 " + leafIndexTemplateList.size() + "个叶子节点：" + leafIndexTemplateListJson);
        int maxLevel = 1; // 用于计算指标得分
        List<LeafIndexPO> leafIndexList = new ArrayList<>();
        for (int i = 0; i < leafIndexTemplateList.size(); i++) {
            AtomicReference<String> scoreExplain = new AtomicReference<>(); // 每个叶子指标下的任务的得分说明一样和叶子指标一致
            IndexTemplatePO leafIndexTemplate = leafIndexTemplateList.get(i);
            String indexId = leafIndexTemplate.getIndexId();
            String parentId = leafIndexTemplate.getParentId(); // 父 id
            String rootId = leafIndexTemplate.getRootId(); // 包 id
            String weight = leafIndexTemplate.getWeight(); // 权重
            Integer packageLevel = leafIndexTemplate.getPackageLevel(); // 几级指标
            String ruleName = leafIndexTemplate.getRuleName();    // 打分脚本名称，例如 AEB_1-1
            String ruleDetails = leafIndexTemplate.getRuleDetails();    // 打分脚本内容
            if (packageLevel > maxLevel) {
                maxLevel = packageLevel;
            }
            log.info("TaskService--state 开始执行对第 " + (i + 1) + " 个叶子节点 " + indexId + " 进行打分！");

            String ruleFilePath = pyPath + "scripts/" + ruleName.split("_")[0] + "/" + ruleName + ".py";
            log.info("TaskService--state 将叶子节点 " + indexId + " 对应的打分规则保存到临时目录：" + ruleFilePath);
            FileUtil.writeInputStreamToLocalFile(IoUtil.stringToInputStream(ruleDetails), ruleFilePath);
            List<TaskPO> taskListOfLeafIndex = taskList.stream()
                    .filter(task -> indexId.equals(task.getLastTargetId()))
                    .collect(Collectors.toList());
            log.info("TaskService--state 叶子节点 " + indexId + " 包括 " + taskListOfLeafIndex.size() + " 个任务！");
            // 计算叶子指标的得分
            taskListOfLeafIndex.forEach(taskOfLeaf -> {
                String task2Id = taskOfLeaf.getId();
                try {
                    String runState = taskOfLeaf.getRunState();
                    if (DictConstants.TASK_ANALYSIS.equals(runState)) {
                        taskMapper.updateSuccessStateWithStopTime(task2Id, DictConstants.TASK_ANALYSING, TimeUtil.getNowForMysql());
                        // 计算每个任务的得分
                        String result1OfMinio = taskOfLeaf.getRunResultFilePath() + "/Ego.csv";
                        String result1OfLinux = linuxTempPath + result1OfMinio;
                        String result2OfMinio = taskOfLeaf.getRunResultFilePath() + "/evaluation.csv";
                        String result2OfLinux = linuxTempPath + result2OfMinio;
                        String scoreCommand = "python3 " + pyPath + "main.py "
                                + result1OfLinux + " "
                                + result2OfLinux + " "
                                + taskOfLeaf.getSceneType() + " " + ruleName; // 指定打分脚本
                        String scoreResult;
                        ScoreTO score;
                        try {
                            log.info("TaskService--state 下载 minio 上的结果文件 " + result1OfMinio + " 和 " + result2OfMinio + " 到临时目录：" + linuxTempPath);
                            MinioUtil.downloadToFile(minioClient, bucketName, result1OfMinio, result1OfLinux);  // 也可改成下载到指定ip的服务器上，需要保证和打分脚本在一台机器上。
                            MinioUtil.downloadToFile(minioClient, bucketName, result2OfMinio, result2OfLinux);  // 也可改成下载到指定ip的服务器上，需要保证和打分脚本在一台机器上。
                        } catch (Exception e) {
                            throw new RuntimeException("------- TaskService--state 下载 minio 上的结果文件出错：" + e.getMessage());
                        }
                        try {
                            log.info("TaskService--state 开始执行打分命令：" + scoreCommand);
                            scoreResult = SshUtil.execute(session, scoreCommand);
                            log.info("TaskService--state 项目" + projectId + " 的任务 " + task2Id + " 打分结束，结果为：" + scoreResult);
                            String replace = StringUtil.replace(scoreResult, "'", "\"");
                            score = JsonUtil.jsonToBean(replace, ScoreTO.class);
                            FileUtil.rm(result1OfLinux);
                            FileUtil.rm(result2OfLinux);
                            log.info("TaskService--state 已删除运行结果文件 Ego.csv 和 evaluation.csv。");
                        } catch (IOException e) {
                            throw new RuntimeException("------- TaskService--state 项目" + projectId + "的任务" + task2Id + " 打分出错，命令为：" + scoreCommand + " 修改状态为：" + DictConstants.TASK_ABORTED + "\n" + e.getMessage());
                        }
                        taskOfLeaf.setReturnSceneId(score.getUnit_scene_ID());
                        taskOfLeaf.setScore(score.getUnit_scene_score());
                        taskOfLeaf.setTargetEvaluate(score.getEvaluate_item());
                        taskOfLeaf.setScoreExplain(score.getScore_description());
                        taskOfLeaf.setModifyUserId(userId);
                        taskOfLeaf.setModifyTime(TimeUtil.getNowForMysql());
                        scoreExplain.set(score.getScore_description());
                        taskOfLeaf.setRunState(DictConstants.TASK_COMPLETED);
                        taskMapper.updateSuccessStateAndScoreResultWithStopTime(taskOfLeaf, DictConstants.TASK_COMPLETED, TimeUtil.getNowForMysql());
                    }
                } catch (Exception e) {
                    taskOfLeaf.setRunState(DictConstants.TASK_ABORTED);
                    taskOfLeaf.setScore(0.0);
                    taskMapper.updateFailStateWithStopTime(task2Id, DictConstants.TASK_ABORTED, TimeUtil.getNowForMysql(), DictConstants.TASK_ERROR_REASON_5);
                    log.error("TaskManager--score 任务 " + task2Id + " 打分出错！", e.getMessage());
                    // 如果打分失败则开始下一个打分
                }
            });


            // 计算不合格的任务数（不到100分就是不合格，执行失败的不算）
            // 计算叶子指标下任务得分总和
            int errorSceneNumber = 0;
            int notScoredSceneNumber = 0;
            int notStandardSceneNumber = 0;
            int standardSceneNumber = 0;
            double leafSum = 0.0;
            for (TaskPO task : taskListOfLeafIndex) {
                Double scoreTemp = task.getScore();
                if (scoreTemp == null) {   // 失败状态的任务是没有分数的，计为 0 分。
                    errorSceneNumber++;
                    scoreTemp = 0.0;
                } else if ("-1".equals(task.getScore().toString())) {  // 如果得分是 -1 ，说明是评分规则无法打分，即也不参与计分
                    notScoredSceneNumber++;
                    continue;
                } else if (scoreTemp < 100.0) {
                    notStandardSceneNumber++;
                } else if (scoreTemp == 100.0) {
                    standardSceneNumber++;
                }

                // 计算分数总和
                leafSum += scoreTemp;
            }

            // 计算任务的个数
            long taskNumberToScore = errorSceneNumber + notStandardSceneNumber + standardSceneNumber;
            log.info("TaskManager--score 项目 " + projectId + " 的叶子指标 " + indexId
                    + " 下参与计算的任务总数为 " + taskNumberToScore
                    + "：仿真异常场景个数 " + errorSceneNumber
                    + "、未达标场景个数 " + notStandardSceneNumber
                    + "、达标场景个数 " + standardSceneNumber
            );
            // 计算叶子指标得分（任务得分总和 / 任务数量）
            double leafIndexScore = taskNumberToScore == 0 ? 0 : NumberUtil.cut(leafSum / taskNumberToScore, 2);
            // 创建叶子指标对象
            leafIndexTemplate.setTempScore(leafIndexScore);

            LeafIndexPO leafIndex = LeafIndexPO.builder()
                    .id(StringUtil.getRandomUUID())
                    .pId(projectId)
                    .target(leafIndexTemplate.getIndexId())
                    .errorSceneNum(errorSceneNumber)
                    .notScoredSceneNum(notScoredSceneNumber)
                    .notStandardSceneNum(notStandardSceneNumber)
                    .standardSceneNum(standardSceneNumber)
                    .score(leafIndexScore)
                    .indexId(indexId)
                    .parentId(parentId)
                    .rootId(rootId)
                    .weight(weight)
                    .scoreExplain(scoreExplain.get())
                    .packageLevel(packageLevel)
                    .build();
            leafIndex.setCreateUserId(userId);
            leafIndex.setCreateTime(TimeUtil.getNowForMysql());
            leafIndex.setModifyUserId(userId);
            leafIndex.setModifyTime(TimeUtil.getNowForMysql());
            leafIndex.setIsDeleted("0");

            leafIndexList.add(leafIndex);
        }
        // 保存叶子指标得分
        taskIndexManager.batchInsertLeafIndex(leafIndexList);
        // 保存一级指标分数
        log.info("TaskManager--score 项目 " + projectId + " 的所有任务分数为：" + taskList);
        computeFirst(leafIndexList, allIndexTemplateList, projectId, maxLevel);


        log.info("TaskManager--score 项目 " + projectId + " 打分完成！");
    }

    public void computeFirst(List<LeafIndexPO> leafIndexList, List<IndexTemplatePO> allIndexTemplateList, String projectId, int maxLevel) {

        log.info("------- /state computeFirst 计算父指标得分：" + leafIndexList);
        Iterator<LeafIndexPO> leafTaskIndexIterator = leafIndexList.iterator();
        // 把 1 级的指标得分直接保存
        while (leafTaskIndexIterator.hasNext()) {
            LeafIndexPO leafTaskIndex = leafTaskIndexIterator.next();
            if (leafTaskIndex.getPackageLevel() == 1) {
                leafTaskIndex.setCreateUserId(leafTaskIndex.getCreateUserId());
                leafTaskIndex.setCreateTime(TimeUtil.getNowForMysql());
                leafTaskIndex.setModifyUserId(leafTaskIndex.getModifyUserId());
                leafTaskIndex.setModifyTime(TimeUtil.getNowForMysql());
                leafTaskIndex.setIsDeleted("0");
                indexMapper.insertFirstIndex(leafTaskIndex);
                leafTaskIndexIterator.remove();
            }
        }
        if (leafIndexList.size() > 0) {
            List<LeafIndexPO> nextLevelIndexList = new ArrayList<>();
            // 找出等级和 maxLevel 不相同的指标暂时不计算
            leafIndexList.stream()
                    .filter(po -> maxLevel != po.getPackageLevel())
                    .forEach(nextLevelIndexList::add);
            // 找出等级和 maxLevel 相同的指标并根据父指标分组
            Map<String, List<LeafIndexPO>> sonTaskIndexMap = leafIndexList.stream()
                    .filter(po -> maxLevel == po.getPackageLevel())
                    .collect(Collectors.groupingBy(LeafIndexPO::getParentId));
            Set<String> parentIdSet = sonTaskIndexMap.keySet();
            List<String> parentIdList = CollectionUtil.setToList(parentIdSet);

            List<IndexTemplatePO> parentIndexTemplateList = allIndexTemplateList.stream()
                    .filter(indexTemplate -> parentIdList.contains(indexTemplate.getIndexId()))
                    .collect(Collectors.toList());
            // 计算父指标得分
            parentIndexTemplateList.forEach(indexTemplate -> {
                String weight = indexTemplate.getWeight();
                List<LeafIndexPO> sonTaskIndexList = sonTaskIndexMap.get(indexTemplate.getIndexId());
                double parentScore = sonTaskIndexList.stream().mapToDouble(taskIndex -> taskIndex.getScore() * Double.parseDouble(taskIndex.getWeight()) / 100).sum();
                LeafIndexPO parentTaskIndex = LeafIndexPO.builder()
                        .id(StringUtil.getRandomUUID())
                        .pId(projectId)
                        .target(indexTemplate.getIndexId())
                        .score(parentScore)
                        .indexId(indexTemplate.getIndexId())
                        .parentId(indexTemplate.getParentId())
                        .rootId(indexTemplate.getRootId())
                        .weight(weight)
                        .packageLevel(maxLevel - 1)
                        .build();
                nextLevelIndexList.add(parentTaskIndex);
            });
            // 将父指标作为叶子指标递归
            computeFirst(nextLevelIndexList, allIndexTemplateList, projectId, maxLevel - 1);
        }
    }

    @SneakyThrows
    public void evaluationLevel(String projectId) {
        String tokenUrl = tokenUri + "?grant_type=client_credentials"
                + "&client_id=" + clientId
                + "&client_secret=" + clientSecret;
        log.info("TaskService--state 获取仿真云平台 token：" + tokenUrl);
        String response = HttpUtil.get(closeableHttpClient, requestConfig, tokenUrl);
        ObjectMapper objectMapper = new ObjectMapper();
        JsonNode jsonNode = objectMapper.readTree(response);
        String accessToken = jsonNode.path("access_token").asText();
        log.info("TaskService--state 仿真云平台 token 为：" + accessToken);
        Map<String, String> headers = new HashMap<>();
        headers.put("Authorization", "Bearer " + accessToken);
        Map<String, String> params = new HashMap<>();
        params.put("id", projectId);
        String post = HttpUtil.post(closeableHttpClient, requestConfig, evaluationLevelUri, headers, params);
        log.info("TaskService--state 访问仿真云平台评价等级接口：" + evaluationLevelUri + "，请求头为：" + headers + "，请求体为：" + params + "结果为：" + post);
    }


    public Boolean taskConfirm(String taskId) {
        // 查询 task 如果不是 pending 则不执行
        String state = taskMapper.selectStateById(taskId);
        return DictConstants.TASK_PENDING.equals(state);
    }

    public void taskTick(String taskId) {
        log.info("TaskService--taskTick 任务 " + taskId + "心跳！");
        TaskPO taskPO = taskMapper.selectById(taskId);
        String projectId = taskPO.getPId();
        String userId = taskPO.getCreateUserId();
        // 刷新 redis 心跳时间
        PrefixTO redisPrefix = projectUtil.getRedisPrefixByUserIdAndProjectIdAndTaskId(userId, projectId, taskId);
        if (RedisUtil.getStringByKey(stringRedisTemplate, redisPrefix.getProjectRunningKey()) != null) {
            stringRedisTemplate.opsForValue().set(redisPrefix.getTaskTickKey(), TimeUtil.getNowString());
        }

    }


    @SneakyThrows
    public void done(PrefixTO redisPrefix, SshClient sshClient, ClientSession clientSession, String projectId, String projectType) {

        // 关闭 ssh 连接
        clientSession.close();
        sshClient.stop();

        // 更新项目状态为已完成
        if (DictConstants.PROJECT_TYPE_MANUAL.equals(projectType)) {
            manualProjectMapper.updateProjectState(projectId, DictConstants.PROJECT_COMPLETED, TimeUtil.getNowForMysql());
        } else if (DictConstants.PROJECT_TYPE_AUTO_SUB.equals(projectType)) {
            autoSubProjectMapper.updateProjectState(projectId, DictConstants.PROJECT_COMPLETED, TimeUtil.getNowForMysql());
        }


        // 删除 kafka topic
//        SshClient clientKafka = SshUtil.getClient();
//        ClientSession sessionKafka = SshUtil.getSession(clientKafka, hostnameKafka, usernameKafka, passwordKafka);
//        String topicDeleteCommand = StringUtil.replace(kafkaDeleteCommand, "topicName", projectId);
//        SshUtil.execute(sessionKafka, topicDeleteCommand);
//        SshUtil.stop(clientKafka, sessionKafka);


        // 归还并行度
        Set<String> nodeOfPodKeySet = RedisUtil.getKeySetByPrefix(stringRedisTemplate, "pod:project-" + projectId);
        for (String nodeOfPodKey : nodeOfPodKeySet) {
            String podName = nodeOfPodKey.split(":")[1];
            String nodeName = projectUtil.getNodeNameOfPod(podName);
            // 删除 pod
            projectUtil.deletePod(podName);
            // 节点并行度加一
            projectUtil.addOneParallelismToNode(nodeName);
        }

        // 删除 redis 中的 项目运行信息 键值对
        RedisUtil.deleteByPrefix(stringRedisTemplate, redisPrefix.getProjectRunningKey());

        if (DictConstants.PROJECT_TYPE_MANUAL.equals(projectType)) {
            log.info("TaskManager--done 手动运行项目 " + projectId + " 执行完成！");
        } else if (DictConstants.PROJECT_TYPE_AUTO_SUB.equals(projectType)) {
            log.info("TaskManager--done 自动运行子项目 " + projectId + " 执行完成！");
        }


    }


}
