package com.css.simulation.resource.algorithm.ctrl;


import api.common.pojo.common.ResponseBodyVO;
import api.common.pojo.constants.LogConstants;
import api.common.pojo.param.algorithm.AlgorithmParameter;
import com.css.simulation.resource.algorithm.service.AlgorithmService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.security.KeyManagementException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;

/**
 * 算法库模块
 */
@Controller
@RequestMapping(value = "/algorithm",name = LogConstants.MODULE_ALGORITH)
public class AlgorithmCtrl {

    @Autowired
    private AlgorithmService service;


    /**
     * 新增/修改算法
     * @param param
     * @return
     */
    @RequestMapping("addOrUpdate")
    @ResponseBody
    public ResponseBodyVO addOrUpdate(@RequestBody AlgorithmParameter param) throws IOException {
        return service.addOrUpdate(param);
    }


    /**
     * 查询算法列表-私有导入和私有仓库页面
     * @param param
     * @return
     */
    @RequestMapping("selectAlgorithmList")
    @ResponseBody
    public ResponseBodyVO selectAlgorithmList(@RequestBody AlgorithmParameter param){
        return service.selectAlgorithmList(param);
    }

    /**
     * 查询算法列表-算法平台页面
     * @param param
     * @return
     */
    @RequestMapping("selectAlgoPlatformList")
    @ResponseBody
    public ResponseBodyVO selectAlgoPlatformList(@RequestBody AlgorithmParameter param){
        return service.selectAlgoPlatformList(param);
    }


    /**
     * 查询算法列表-公有页面
     * @param param
     * @return
     */
    @RequestMapping("selectSharedAlgorithmList")
    @ResponseBody
    public ResponseBodyVO selectSharedAlgorithmList(@RequestBody AlgorithmParameter param){
        return service.selectSharedAlgorithmList(param);
    }

     /**
     * 删除算法
     * @return
     */
    @RequestMapping("deleteByid")
    @ResponseBody
    public ResponseBodyVO deleteByid(@RequestBody AlgorithmParameter param){
        return service.deleteByid(param);
    }


    /**
     * 分享算法
     * @return
     */
    @RequestMapping("shareAlgorithm")
    @ResponseBody
    public ResponseBodyVO shareAlgorithm(@RequestBody AlgorithmParameter param){
        return service.shareAlgorithm(param);
    }


    /**
     * 查看算法详情
     * @param param
     * @return
     */
    @RequestMapping("selectDetailsById")
    @ResponseBody
    public ResponseBodyVO selectDetailsById(@RequestBody AlgorithmParameter param){
        String algorithmId = param.getId();
        return service.selectDetailsById(algorithmId);
    }

    /**
     * git测试连接
     */
    @RequestMapping("testConnection")
    @ResponseBody
    public ResponseBodyVO<String> testConnection(@RequestBody AlgorithmParameter param){
        return service.testConnection(param);
    }

    /**
     * 根据git仓库地址获取版本号
     */
    @RequestMapping("getGitVersion")
    @ResponseBody
    public ResponseBodyVO getGitVersion(@RequestBody AlgorithmParameter param) throws IOException, NoSuchAlgorithmException, KeyStoreException, KeyManagementException {
        String id = param.getId();
        String gitVersion = service.getGitVersion(id);
        return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS,"请求成功", gitVersion);
    }

}
