package com.css.simulation.resource.home.service;

import api.common.pojo.common.PageVO;
import api.common.pojo.common.ResponseBodyVO;
import api.common.pojo.po.home.SystemServerPO;
import api.common.pojo.vo.home.*;
import api.common.util.ObjectUtil;
import api.common.util.TimeUtil;
import com.css.simulation.resource.common.utils.AuthUtil;
import com.css.simulation.resource.common.utils.PageUtil;
import com.css.simulation.resource.feign.OauthClientService;
import com.css.simulation.resource.log.mapper.LogMapper;
import com.css.simulation.resource.home.mapper.HomePageMapper;
import com.css.simulation.resource.system.mapper.UserMapper;
import com.github.pagehelper.PageInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class HomePageService {

    @Resource
    HomePageMapper homePageMapper;

    @Autowired
    UserMapper userMapper;

    @Autowired
    LogMapper logMapper;

    @Resource
    OauthClientService oauthClientService;


    public ResponseBodyVO<HardwareVO> selectHardware() {
        List<SystemServerPO> systemServerPOList = homePageMapper.selectAll();
        long memorySum = systemServerPOList.stream().mapToLong(SystemServerPO::getMemoryTotal).sum();  // 内存之和
        long diskSum = systemServerPOList.stream().mapToLong(SystemServerPO::getDiskTotal).sum();  // 磁盘之和
        long cpuSum = systemServerPOList.stream().mapToLong(SystemServerPO::getCpuTotal).sum();  // cpu线程数 之和
        long gpuSum = systemServerPOList.stream().mapToLong(SystemServerPO::getGpuTotal).sum();  // 显存之和
        HardwareVO build = HardwareVO.builder()
                .serverNumber(systemServerPOList.size())
                .memoryUsage((int) systemServerPOList.stream().mapToDouble(po -> po.getMemoryUsage() * po.getMemoryTotal() * 1.0 / memorySum).sum() + "%")
                .diskUsage((int) systemServerPOList.stream().mapToDouble(po -> po.getDiskUsage() * po.getDiskTotal() * 1.0 / diskSum).sum() + "%")
                .cpuUsage((int) systemServerPOList.stream().mapToDouble(po -> po.getCpuUsage() * po.getCpuTotal() * 1.0 / cpuSum).sum() + "%")
                .gpuUsage((int) systemServerPOList.stream().mapToDouble(po -> po.getGpuUsage() * po.getGpuTotal() * 1.0 / gpuSum).sum() + "%")
                .build();
        return new ResponseBodyVO<>(ResponseBodyVO.Response.SUCCESS, build);
    }


    public ResponseBodyVO<ServiceVO> selectService() {
        List<SystemServerPO> systemServerPOList = homePageMapper.selectAll();
        long count = systemServerPOList.stream().filter(system -> "gpu".equals(system.getServerType())).count();
        ServiceVO build = ServiceVO.builder()
                .userNumber(userMapper.selectCount())
                .onlineNumber(oauthClientService.online().getInfo())
                .simulationNodeNumber((int) count)
                .simulationLicenseNumber(homePageMapper.selectSumOfTaskNumber())
                .dynamicsLicenseNumber(0)   // 待定
                .build();
        return new ResponseBodyVO<>(ResponseBodyVO.Response.SUCCESS, build);
    }

    public ResponseBodyVO<PageInfo<SystemServerVO>> selectServer(PageVO pageVO) {
        PageUtil.setPageInfo(pageVO);
        List<SystemServerPO> systemServerPOList = homePageMapper.selectAll();
        List<SystemServerVO> systemServerVOList = new ArrayList<>();
        systemServerPOList.forEach(po -> {
            SystemServerVO vo = SystemServerVO.builder()
                    .id(po.getId())
                    .serverId(po.getServerId())
                    .serverAddress(po.getServerAddress())
                    .serverType(po.getServerType())
                    .cpuUsage((int) (po.getCpuUsage() * 100))
                    .memoryUsage((int) (po.getMemoryUsage() * 100))
                    .memoryAvailable((double) (int) (po.getMemoryAvailable() * 1.0 / 1024.0 / 1024.0 * 100) / 100)
                    .diskUsage((int) (po.getDiskUsage() * 100))
                    .diskAvailable((double) (int) (po.getDiskAvailable() * 1.0 / 1024.0 / 1024.0 * 100) / 100)
                    .taskNumber(po.getTaskNumber())
                    .build();
            systemServerVOList.add(vo);
        });
        return new ResponseBodyVO<>(ResponseBodyVO.Response.SUCCESS, new PageInfo<>(systemServerVOList));
    }

    public List<LineChartVO> selectAccess() {
        String dateBegin = TimeUtil.getMinusDaysStr(6);
        List<LineChartVO> listVo = logMapper.getAccessCount(dateBegin);
        List<LineChartVO> list = makeLineChartDate(listVo, 6);
        return list;
    }

    public List<LineChartVO> selectRunTaskChart() {
        Map<String, String> param = new HashMap();
        param.put("dateBegin", TimeUtil.getMinusDaysStr(6));
        param.put("createUserId", AuthUtil.getCurrentUserId());
        List<LineChartVO> listVo = homePageMapper.selectRunTaskChart(param);
        List<LineChartVO> list = makeLineChartDate(listVo, 6);
        return list;
    }

    public List<LineChartVO> selectRunProjectChart() {
        Map<String, String> param = new HashMap();
        param.put("dateBegin", TimeUtil.getMinusDaysStr(6));
        param.put("createUserId", AuthUtil.getCurrentUserId());
        List<LineChartVO> listVo = homePageMapper.selectRunProjectChart(param);
        List<LineChartVO> list = makeLineChartDate(listVo, 6);
        return list;
    }

    /**
     * 构建图标基础数据
     */
    private List<LineChartVO> makeLineChartDate(List<LineChartVO> listVo, int j) {
        List<LineChartVO> list = new ArrayList<>();
        boolean notNull = ObjectUtil.isNotNull(listVo);
        for (int i = j; i >= 0; i--) {
            String date = TimeUtil.getMinusDaysStr(i);
            if (notNull) {
                List<LineChartVO> collect = listVo.stream().filter(v -> date.equals(v.getToDate())).collect(Collectors.toList());
                if (ObjectUtil.isNotNull(collect)) {
                    LineChartVO vo = collect.get(0);
                    list.add(vo);
                    continue;
                }
            }
            //创建新数据
            LineChartVO vo = new LineChartVO();
            vo.setToDate(date);
            vo.setNum(0);
            list.add(vo);
        }
        return list;
    }

    public Integer getRunningTask() {
        return homePageMapper.getRunningTask(AuthUtil.getCurrentUserId());
    }
}
