package com.css.simulation.resource.model.ctrl;

import api.common.pojo.common.ResponseBodyVO;
import api.common.pojo.constants.LogConstants;
import api.common.pojo.param.model.CameraParam;
import api.common.pojo.po.model.CameraPO;
import api.common.pojo.vo.model.CameraVO;
import api.common.util.ObjectUtil;
import com.css.simulation.resource.model.service.CameraService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import java.util.List;

@Controller
@RequestMapping(value = "/camera" , name = LogConstants.MODULE_CAMERA)
public class CameraCtrl {

    @Autowired
    CameraService cameraService;

    /**
     * 获取列表
     */
    @RequestMapping("/getCameraList")
    @ResponseBody
    public ResponseBodyVO<List<CameraVO>> getCameraList(@RequestBody CameraParam cameraParam){
        ResponseBodyVO<List<CameraVO>> response = new ResponseBodyVO<List<CameraVO>>(ResponseBodyVO.Response.SUCCESS);
        response.setInfo(cameraService.getCameraList(cameraParam));
        return response;
    }

    /**
     * 获取私有列表
     */
    @RequestMapping("/getMyCameraList")
    @ResponseBody
    public ResponseBodyVO<List<CameraVO>> getMyCameraList(@RequestBody CameraParam cameraParam){
        ResponseBodyVO<List<CameraVO>> response = new ResponseBodyVO<List<CameraVO>>(ResponseBodyVO.Response.SUCCESS);
        response.setInfo(cameraService.getMyCameraList(cameraParam));
        return response;
    }


    /**
     * 根据id获取详情
     */
    @RequestMapping("/getCameraInfo")
    @ResponseBody
    public ResponseBodyVO<CameraVO> getCameraInfo(@RequestBody CameraParam cameraParam){
        ResponseBodyVO<CameraVO> response = new ResponseBodyVO<CameraVO>(ResponseBodyVO.Response.SUCCESS);
        response.setInfo(cameraService.getCameraInfo(cameraParam));
        return response;
    }

    /**
     * 新增、修改、另存为
     */
    @RequestMapping("/saveCamera")
    @ResponseBody
    public ResponseBodyVO<CameraPO> saveCamera(@RequestBody CameraPO cameraPO){
        if(ObjectUtil.isNull(cameraPO)){
            return new ResponseBodyVO(false, 500, "参数必传！",null);
        }
        ResponseBodyVO<CameraPO> response = new ResponseBodyVO<CameraPO>(ResponseBodyVO.Response.SUCCESS);
        CameraPO po = cameraService.saveCamera(cameraPO);
        if(ObjectUtil.isNull(po.getId())){
            return new ResponseBodyVO(false, 500, "传感器名称重复！",null);
        }
        response.setInfo(po);
        return response;
    }

    /**
     * 分享
     */
    @RequestMapping("/shareCamera")
    @ResponseBody
    @PreAuthorize("@AuthorityCheck.admin()")
    public ResponseBodyVO<CameraPO> shareCamera(@RequestBody CameraPO cameraPO){
        if(ObjectUtil.isNull(cameraPO)){
            return new ResponseBodyVO(false, 500, "参数必传！",null);
        }
        ResponseBodyVO<CameraPO> response = new ResponseBodyVO<CameraPO>(ResponseBodyVO.Response.SUCCESS);
        CameraPO po = cameraService.shareCamera(cameraPO);
        if(ObjectUtil.isNull(po.getId())){
            return new ResponseBodyVO(false, 500, "传感器名称重复！",null);
        }
        response.setInfo(po);
        return response;
    }

    /**
     * 删除
     */
    @RequestMapping("/delCameraById")
    @ResponseBody
    public ResponseBodyVO delCameraById(@RequestBody CameraPO cameraPO){
        int i = cameraService.delCameraById(cameraPO);
        if(i>0){
            return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS);
        }else{
            return new ResponseBodyVO(false, 500, "删除失败！",null);
        }
    }
}
