package com.css.simulation.resource.model.service;

import api.common.pojo.constants.DictConstants;
import api.common.pojo.param.model.ConfigPageParam;
import api.common.pojo.po.model.ConfigPO;
import api.common.pojo.po.model.ConfigSensorPO;
import api.common.pojo.vo.model.ConfigSensorVO;
import api.common.pojo.vo.model.ConfigVO;
import api.common.util.LogUtil;
import api.common.util.ObjectUtil;
import api.common.util.StringUtil;
import api.common.util.TimeUtil;
import com.css.simulation.resource.common.utils.AuthUtil;
import com.css.simulation.resource.common.utils.PageUtil;
import com.css.simulation.resource.model.mapper.CameraMapper;
import com.css.simulation.resource.model.mapper.ConfigMapper;
import com.github.pagehelper.PageInfo;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.sql.Timestamp;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class ConfigService {

    @Autowired
    ConfigMapper configMapper;

    public PageInfo<ConfigVO> getConfigPageList(ConfigPageParam pageParam) {
        PageUtil.setPageInfo(pageParam);
        pageParam.setCreateUserId(AuthUtil.getCurrentUserId());
        if(DictConstants.YES.equals(pageParam.getShare())){//公有查询去掉创建人id条件
            pageParam.setCreateUserId(null);
        }
        List<ConfigVO> list = configMapper.getConfigPageList(pageParam);
        //传感器数量
        list.forEach(configVO -> {
            List<ConfigSensorVO> sensorList = configMapper.getConfigSensors(configVO);
            Map<String, List<ConfigSensorVO>> configSensors = sensorList.stream().collect(Collectors.groupingBy(ConfigSensorVO::getSensorType));
            Map<String,String> sensorCount = new HashMap<>();
            configSensors.forEach((sensorType,configSensorVOs)->{
                sensorCount.put(sensorType,String.valueOf(configSensorVOs.size()));
            });
            configVO.setSensorCount(sensorCount);
        });
        return new PageInfo<>(list);
    }

    public ConfigVO getConfigInfo(ConfigVO configVO) {
        configVO = configMapper.getConfigInfo(configVO);
        List<ConfigSensorVO> sensorList = configMapper.getConfigSensors(configVO);
        Map<String, List<ConfigSensorVO>> configSensors = sensorList.stream().collect(Collectors.groupingBy(ConfigSensorVO::getSensorType));
        configVO.setConfigSensors(configSensors);
        return configVO;
    }

    public ConfigPO saveConfig(ConfigVO configVO) {
        //构建主表对象
        ConfigPO configPO = new ConfigPO();
        String currentUserId = AuthUtil.getCurrentUserId();
        Timestamp currentTime = TimeUtil.getNowForMysql();
        ObjectUtil.voToPo(configVO,configPO);
        configPO.setCreateUserId(currentUserId);
        configPO.setCreateTime(currentTime);
        configPO.setModifyUserId(currentUserId);
        configPO.setModifyTime(currentTime);
        configPO.setIsDeleted(DictConstants.NO);
        configPO.setShare(DictConstants.NO);//私有
        //单独处理配置描述
        configPO.setDescription(configVO.getConfigDescription());
        //名称校验
        List<ConfigVO> list = configMapper.checkConfigName(configPO);
        if(ObjectUtil.isNotNull(list)){
            configPO.setId(null);
            return configPO;
        }
        //主表主键
        String configId = configPO.getId();
        if(ObjectUtil.isNull(configId)){//新增
            configId = StringUtil.getRandomUUID();
            configPO.setId(configId);
            configPO.setConfigCode(StringUtil.getRandomCode());
            configMapper.insertConfig(configPO);
            LogUtil.insert();
        }else{//修改
            configMapper.updateConfig(configPO);
            LogUtil.update();
        }
        //删除子表旧数据
        configMapper.delConfigSensors(configPO);
        //构建字表新数据
        //List<ConfigSensorVO> configSensorVOs = configVO.getConfigSensors();
        Map<String, List<ConfigSensorVO>> configSensors = configVO.getConfigSensors();
        List<ConfigSensorPO> configSensorPOs = new ArrayList<>();
        if(ObjectUtil.isNotNull(configSensors)){
            configSensors.forEach((sensorType,configSensorVOs)->{
                if(ObjectUtil.isNotNull(configSensorVOs)){
                    configSensorVOs.forEach(vo -> {
                        ConfigSensorPO po = new ConfigSensorPO();
                        ObjectUtil.voToPo(vo,po);
                        po.setId(StringUtil.getRandomUUID());
                        //子表外键
                        po.setConfigId(configPO.getId());
                        po.setCreateUserId(currentUserId);
                        po.setCreateTime(currentTime);
                        po.setModifyUserId(currentUserId);
                        po.setModifyTime(currentTime);
                        po.setIsDeleted(DictConstants.NO);
                        configSensorPOs.add(po);
                    });
                }
            });
        }
        if(ObjectUtil.isNotNull(configSensorPOs)){
            configMapper.insertConfigSensors(configSensorPOs);
        }
        return configPO;
    }

    public int delConfigById(ConfigVO vo) {
        ConfigPO po = new ConfigPO();
        ObjectUtil.voToPo(vo,po);
        po.setIsDeleted(DictConstants.YES);
        po.setModifyUserId(AuthUtil.getCurrentUserId());
        po.setModifyTime(TimeUtil.getNowForMysql());
        int i = configMapper.delConfigById(po);
        LogUtil.delete();
        return i;
    }

    public ConfigPO shareConfigByInfo(ConfigVO configVO) {
        //构建主表对象
        ConfigPO configPO = new ConfigPO();
        ObjectUtil.voToPo(configVO,configPO);
        configPO.setShare(DictConstants.YES);//转公有
        configPO.setCreateUserId(null);
        //名称校验
        List<ConfigVO> list = configMapper.checkConfigName(configPO);
        if(ObjectUtil.isNotNull(list)){
            configPO.setId(null);
            return configPO;
        }
        String currentUserId = AuthUtil.getCurrentUserId();
        Timestamp currentTime = TimeUtil.getNowForMysql();
        configPO.setCreateUserId(currentUserId);
        configPO.setCreateTime(currentTime);
        configPO.setModifyUserId(currentUserId);
        configPO.setModifyTime(currentTime);
        configPO.setIsDeleted(DictConstants.NO);
        //单独处理配置描述
        configPO.setDescription(configVO.getConfigDescription());
        //主表主键
        String configId = StringUtil.getRandomUUID();
        configPO.setId(configId);
        configPO.setConfigCode(StringUtil.getRandomCode());
        configMapper.insertConfig(configPO);
        //删除子表旧数据
        configMapper.delConfigSensors(configPO);
        //构建字表新数据
        Map<String, List<ConfigSensorVO>> configSensors = configVO.getConfigSensors();
        List<ConfigSensorPO> configSensorPOs = new ArrayList<>();
        if(ObjectUtil.isNotNull(configSensors)){
            configSensors.forEach((sensorType,configSensorVOs)->{
                if(ObjectUtil.isNotNull(configSensorVOs)){
                    configSensorVOs.forEach(vo -> {
                        ConfigSensorPO po = new ConfigSensorPO();
                        ObjectUtil.voToPo(vo,po);
                        po.setId(StringUtil.getRandomUUID());
                        //子表外键
                        po.setConfigId(configPO.getId());
                        po.setCreateUserId(currentUserId);
                        po.setCreateTime(currentTime);
                        po.setModifyUserId(currentUserId);
                        po.setModifyTime(currentTime);
                        po.setIsDeleted(DictConstants.NO);
                        configSensorPOs.add(po);
                    });
                }
            });
        }
        if(ObjectUtil.isNotNull(configSensorPOs)){
            configMapper.insertConfigSensors(configSensorPOs);
        }
        return configPO;
    }

    public ConfigPO shareConfigById(ConfigVO configVO) {
        configVO = getConfigInfo(configVO);
        ConfigPO configPO = shareConfigByInfo(configVO);
        LogUtil.share();
        return configPO;
    }

    public int getConfigTotal(){
        return configMapper.getConfigTotal(AuthUtil.getCurrentUserId());
    }
}
