package com.css.simulation.resource.model.service;

import api.common.pojo.constants.DictConstants;
import api.common.pojo.param.model.LidarParam;
import api.common.pojo.po.model.LidarPO;
import api.common.pojo.vo.model.LidarVO;
import api.common.util.LogUtil;
import api.common.util.ObjectUtil;
import api.common.util.StringUtil;
import api.common.util.TimeUtil;
import com.css.simulation.resource.common.utils.AuthUtil;
import com.css.simulation.resource.model.mapper.LidarMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.sql.Timestamp;
import java.util.List;

@Service
public class LidarService {

    @Autowired
    LidarMapper lidarMapper;

    public List<LidarVO> getLidarList(LidarParam lidarParam) {
        lidarParam.setCreateUserId(AuthUtil.getCurrentUserId());
        lidarParam.setShare(null);
        List<LidarVO> list = lidarMapper.getLidarList(lidarParam);
        return list;
    }

    public List<LidarVO> getMyLidarList(LidarParam lidarParam) {
        lidarParam.setCreateUserId(AuthUtil.getCurrentUserId());
        lidarParam.setShare(DictConstants.NO);
        List<LidarVO> list = lidarMapper.getLidarList(lidarParam);
        return list;
    }

    public LidarVO getLidarInfo(LidarParam lidarParam) {
        return lidarMapper.getLidarInfo(lidarParam);
    }

    public LidarPO saveLidar(LidarPO lidarPO) {
        //常规字段赋值
        String currentUserId = AuthUtil.getCurrentUserId();
        Timestamp currentTime = TimeUtil.getNowForMysql();
        lidarPO.setCreateUserId(currentUserId);
        lidarPO.setCreateTime(currentTime);
        lidarPO.setModifyUserId(currentUserId);
        lidarPO.setModifyTime(currentTime);
        lidarPO.setIsDeleted(DictConstants.NO);
        lidarPO.setShare(DictConstants.NO);//私有
        //名称校验
        List<LidarVO> list = lidarMapper.checkLidarName(lidarPO);
        if(ObjectUtil.isNotNull(list)){
            lidarPO.setId(null);
            return lidarPO;
        }
        String id = lidarPO.getId();
        if(ObjectUtil.isNull(id)){//新增
            lidarPO.setId(StringUtil.getRandomUUID());
            lidarPO.setSensorCode(StringUtil.getRandomCode());
            lidarMapper.insert(lidarPO);
            LogUtil.insert();
        }else{//修改
            lidarMapper.update(lidarPO);
            LogUtil.update();
        }
        return lidarPO;
    }

    public LidarPO shareLidar(LidarPO lidarPO) {
        //调用保存
        saveLidar(lidarPO);
        lidarPO.setShare(DictConstants.YES);//转公有
        lidarPO.setCreateUserId(null);
        //名称校验
        List<LidarVO> list = lidarMapper.checkLidarName(lidarPO);
        if(ObjectUtil.isNotNull(list)){
            lidarPO.setId(null);
            return lidarPO;
        }
        //常规字段赋值
        String currentUserId = AuthUtil.getCurrentUserId();
        Timestamp currentTime = TimeUtil.getNowForMysql();
        lidarPO.setCreateUserId(currentUserId);
        lidarPO.setCreateTime(currentTime);
        lidarPO.setModifyUserId(currentUserId);
        lidarPO.setModifyTime(currentTime);
        lidarPO.setIsDeleted(DictConstants.NO);
        lidarPO.setId(StringUtil.getRandomUUID());
        lidarPO.setSensorCode(StringUtil.getRandomCode());
        lidarMapper.insert(lidarPO);
        LogUtil.share();
        return lidarPO;
    }

    public int delLidarById(LidarPO lidarPO) {
        lidarPO.setIsDeleted(DictConstants.YES);
        lidarPO.setModifyUserId(AuthUtil.getCurrentUserId());
        lidarPO.setModifyTime(TimeUtil.getNowForMysql());
        int i = lidarMapper.delLidarById(lidarPO);
        LogUtil.delete();
        return i;
    }
}
