package com.css.simulation.resource.model.service;


import api.common.pojo.constants.DictConstants;
import api.common.pojo.param.model.OgtParam;
import api.common.pojo.po.model.OgtPO;
import api.common.pojo.vo.model.OgtVO;
import api.common.util.LogUtil;
import api.common.util.ObjectUtil;
import api.common.util.StringUtil;
import api.common.util.TimeUtil;
import com.css.simulation.resource.common.utils.AuthUtil;
import com.css.simulation.resource.model.mapper.OgtMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.sql.Timestamp;
import java.util.List;

@Service
public class OgtService {

    @Autowired
    OgtMapper ogtMapper;


    public List<OgtVO> getOgtList(OgtParam ogtParam) {
        ogtParam.setCreateUserId(AuthUtil.getCurrentUserId());
        ogtParam.setShare(null);
        List<OgtVO> list = ogtMapper.getOgtList(ogtParam);
        return list;
    }

    public List<OgtVO> getMyOgtList(OgtParam ogtParam) {
        ogtParam.setCreateUserId(AuthUtil.getCurrentUserId());
        ogtParam.setShare(DictConstants.NO);
        List<OgtVO> list = ogtMapper.getOgtList(ogtParam);
        return list;
    }

    public OgtVO getOgtInfo(OgtParam ogtParam) {
        return ogtMapper.getOgtInfo(ogtParam);
    }

    public OgtPO saveOgt(OgtPO ogtPO) {
        //常规字段赋值
        String currentUserId = AuthUtil.getCurrentUserId();
        Timestamp currentTime = TimeUtil.getNowForMysql();
        ogtPO.setCreateUserId(currentUserId);
        ogtPO.setCreateTime(currentTime);
        ogtPO.setModifyUserId(currentUserId);
        ogtPO.setModifyTime(currentTime);
        ogtPO.setIsDeleted(DictConstants.NO);
        ogtPO.setShare(DictConstants.NO);//私有
        //名称校验
        List<OgtVO> list = ogtMapper.checkOgtName(ogtPO);
        if(ObjectUtil.isNotNull(list)){
            ogtPO.setId(null);
            return ogtPO;
        }
        String id = ogtPO.getId();
        if(ObjectUtil.isNull(id)){//新增
            ogtPO.setId(StringUtil.getRandomUUID());
            ogtPO.setSensorCode(StringUtil.getRandomCode());
            ogtMapper.insert(ogtPO);
            LogUtil.insert();
        }else{//修改
            ogtMapper.update(ogtPO);
            LogUtil.update();
        }
        return ogtPO;
    }

    public OgtPO shareOgt(OgtPO ogtPO) {
        //调用保存
        saveOgt(ogtPO);
        ogtPO.setShare(DictConstants.YES);//转公有
        ogtPO.setCreateUserId(null);
        //名称校验
        List<OgtVO> list = ogtMapper.checkOgtName(ogtPO);
        if(ObjectUtil.isNotNull(list)){
            ogtPO.setId(null);
            return ogtPO;
        }
        //常规字段赋值
        String currentUserId = AuthUtil.getCurrentUserId();
        Timestamp currentTime = TimeUtil.getNowForMysql();
        ogtPO.setCreateUserId(currentUserId);
        ogtPO.setCreateTime(currentTime);
        ogtPO.setModifyUserId(currentUserId);
        ogtPO.setModifyTime(currentTime);
        ogtPO.setIsDeleted(DictConstants.NO);
        ogtPO.setId(StringUtil.getRandomUUID());
        ogtPO.setSensorCode(StringUtil.getRandomCode());
        ogtMapper.insert(ogtPO);
        LogUtil.share();
        return ogtPO;
    }

    public int delOgtById(OgtPO ogtPO) {
        ogtPO.setIsDeleted(DictConstants.YES);
        ogtPO.setModifyUserId(AuthUtil.getCurrentUserId());
        ogtPO.setModifyTime(TimeUtil.getNowForMysql());
        int i = ogtMapper.delOgtById(ogtPO);
        LogUtil.delete();
        return i;
    }
}
