package com.css.simulation.resource.model.service;

import api.common.pojo.constants.DictConstants;
import api.common.pojo.param.model.VehicleParam;
import api.common.pojo.po.model.VehiclePO;
import api.common.pojo.vo.model.VehicleVO;
import api.common.util.LogUtil;
import api.common.util.ObjectUtil;
import api.common.util.StringUtil;
import com.css.simulation.resource.common.utils.AuthUtil;
import com.css.simulation.resource.common.utils.PoUtil;
import com.css.simulation.resource.model.mapper.VehicleMapper;
import org.apache.tomcat.util.buf.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.RequestBody;

import java.util.Arrays;
import java.util.List;

@Service
public class VehicleService {

    @Autowired
    VehicleMapper vehicleMapper;

    public List<VehicleVO> getVehicleList(@RequestBody VehicleParam vehicleParam) {
        vehicleParam.setCreateUserId(AuthUtil.getCurrentUserId());
        vehicleParam.setShare(null);
        List<VehicleVO> list = vehicleMapper.getVehicleList(vehicleParam);
        return list;
    }

    public List<VehicleVO> getMyVehicleList(VehicleParam vehicleParam) {
        vehicleParam.setCreateUserId(AuthUtil.getCurrentUserId());
        vehicleParam.setShare(DictConstants.NO);
        List<VehicleVO> list = vehicleMapper.getVehicleList(vehicleParam);
        return list;
    }

    public VehicleVO getVehicleInfo(VehicleParam vehicleParam) {
        VehicleVO vehicleInfo = vehicleMapper.getVehicleInfo(vehicleParam);
        //车辆模型字段解析
        String vehicleType = vehicleInfo.getVehicleTypeStr();
        vehicleInfo.setVehicleType(vehicleType.split(","));
        return vehicleInfo;
    }

    public VehiclePO saveVehicle(VehicleVO vehicleVO) {
        VehiclePO vehiclePO = new VehiclePO();
        ObjectUtil.voToPo(vehicleVO,vehiclePO);
        //车辆模型字段解析
        String[] vehicleType = vehicleVO.getVehicleType();
        String vehicleTypeStr = StringUtils.join(Arrays.asList(vehicleType),',');
        vehiclePO.setVehicleType(vehicleTypeStr);
        //常规字段赋值
        PoUtil.initAddPo(vehiclePO);
        vehiclePO.setShare(DictConstants.NO);//私有
        //名称校验
        List<VehicleVO> list = vehicleMapper.checkVehicleName(vehiclePO);
        if(ObjectUtil.isNotNull(list)){
            vehiclePO.setId(null);
            return vehiclePO;
        }
        String id = vehiclePO.getId();
        if(ObjectUtil.isNull(id)){//新增
            vehiclePO.setId(StringUtil.getRandomUUID());
            vehiclePO.setVehicleCode(StringUtil.getRandomCode());
            vehicleMapper.insert(vehiclePO);
            LogUtil.insert();
        }else{//修改
            vehicleMapper.update(vehiclePO);
            LogUtil.update();
        }
        return vehiclePO;
    }

    public VehiclePO shareVehicle(VehicleVO vehicleVO) {
        //调用保存
        VehiclePO vehiclePO = saveVehicle(vehicleVO);
        vehiclePO.setShare(DictConstants.YES);//转公有
        vehiclePO.setCreateUserId(null);
        //名称校验
        List<VehicleVO> list = vehicleMapper.checkVehicleName(vehiclePO);
        if(ObjectUtil.isNotNull(list)){
            vehiclePO.setId(null);
            return vehiclePO;
        }
        //常规字段赋值
        PoUtil.initAddPo(vehiclePO);
        vehiclePO.setId(StringUtil.getRandomUUID());
        vehiclePO.setVehicleCode(StringUtil.getRandomCode());
        vehicleMapper.insert(vehiclePO);
        LogUtil.share();
        return vehiclePO;
    }

    public int delVehicleById(VehiclePO vehiclePO) {
        PoUtil.initDelPo(vehiclePO);
        LogUtil.delete();
        return vehicleMapper.delVehicleById(vehiclePO);
    }
}
