package com.css.simulation.resource.model.service;

import api.common.pojo.constants.LogConstants;
import api.common.pojo.param.model.VehiclePageParam;
import api.common.pojo.po.model.VehicleTempPO;
import api.common.pojo.vo.model.VehicleDefaultValueVO;
import api.common.pojo.vo.model.VehicleTempVO;
import api.common.util.ObjectUtil;
import api.common.util.StringUtil;
import com.css.simulation.resource.common.utils.PageUtil;
import com.css.simulation.resource.common.utils.PoUtil;
import com.css.simulation.resource.log.service.LogService;
import com.css.simulation.resource.model.mapper.VehicleTempMapper;
import com.github.pagehelper.PageInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class VehicleTempService {


    @Autowired
    VehicleTempMapper vehicleTempMapper;

    @Autowired
    LogService logService;

    public List getVehicleTempTree() {
        List<VehicleTempVO> list = vehicleTempMapper.getVehicleTempTree();
        List<Map> treeList = new ArrayList<>();
        Map<String, List<VehicleTempVO>> typeCollect = list.stream().collect(Collectors.groupingBy(VehicleTempVO::getVehicleType,LinkedHashMap::new, Collectors.toList()));
        typeCollect.forEach((typeKey,typeValue)->{
            Map typeMap = new HashMap();
            typeMap.put("code",typeKey);
            typeMap.put("name",typeKey);
            //typeMap.put("name",typeValue.get(0).getVehicleName());
            List<Map> modelList = new LinkedList();
            Map<String, List<VehicleTempVO>> modelCollect = typeValue.stream().collect(Collectors.groupingBy(VehicleTempVO::getVehicleModel,LinkedHashMap::new, Collectors.toList()));
            modelCollect.forEach((modelKey,modelValue)->{
                Map modelMap = new HashMap();
                modelMap.put("code",modelKey);
                modelMap.put("name",modelKey);
                List<Map> colourList = new LinkedList();
                modelValue.forEach(vo->{
                    Map colourMap = new HashMap();
                    colourMap.put("code",vo.getVehicleColour());
                    colourMap.put("name",vo.getVehicleColour());
                    VehicleDefaultValueVO defaultValueVO = new VehicleDefaultValueVO();
                    ObjectUtil.voToPo(vo,defaultValueVO);
                    colourMap.put("vo",defaultValueVO);
                    colourList.add(colourMap);
                });
                modelMap.put("children",colourList);
                modelList.add(modelMap);
            });
            typeMap.put("children",modelList);
            treeList.add(typeMap);
        });
        return treeList;
    }

    public PageInfo<VehicleTempVO> getVehicleTempPageList(VehiclePageParam pageParam) {
        PageUtil.setPageInfo(pageParam);
        List<VehicleTempVO> list = vehicleTempMapper.getVehicleTempPageList(pageParam);
        return new PageInfo<>(list);
    }

    public VehicleTempVO getVehicleTempInfo(VehiclePageParam param) {
        return vehicleTempMapper.getVehicleTempInfo(param);
    }

    public VehicleTempPO saveVehicleTemp(VehicleTempPO po) {
        //名称校验
        List<VehicleTempVO> list = vehicleTempMapper.checkVehicleName(po);
        if(ObjectUtil.isNotNull(list)){
            po.setId(null);
            return po;
        }
        String id = po.getId();
        //常规字段赋值
        PoUtil.initAddPo(po);
        if(ObjectUtil.isNull(id)){//新增
            po.setId(StringUtil.getRandomUUID());
            po.setVehicleCode(StringUtil.getRandomCode());
            po.setValid("1");
            vehicleTempMapper.insert(po);
            logService.logVehicle(LogConstants.SYS_LOG_VEHICLE_INSERT,po);
        }else{//修改
            vehicleTempMapper.update(po);
            logService.logVehicle(LogConstants.SYS_LOG_VEHICLE_UPDATE,po);
        }
        return po;
    }

    public int delVehicleTempById(VehicleTempPO po) {
        PoUtil.initDelPo(po);
        logService.logVehicle(LogConstants.SYS_LOG_VEHICLE_DEL,po);
        return vehicleTempMapper.delVehicleTempById(po);
    }
}
